#include </usr/include/gtk-2.0/gtk/gtk.h>
#include <unistd.h>
#include <stdlib.h>
#include <string.h>

/*
// Xfce Wallpaper Changer Non-Mono Edition
// by Gene Jones
//
// v1.0 - November 8, 2009
// 
// Usage: Set desktop background in Xfce to an Image List, populate the list, run this app.
//
// Thanks to: ZetCode, GtkForums, Gnu, Gtk+, SciTE, Xfce and Google


/* Declarations */
static void traySwitch(GtkMenuItem *item, gpointer user_data);
static void trayExit(GtkMenuItem *item, gpointer user_data);
static void trayIconActivated(GObject *trayIcon, gpointer data);
static void trayIconPopup(GtkStatusIcon *status_icon, guint button, guint32 activate_time, gpointer popUpMenu);
static void destroy (GtkWidget*, gpointer);
static gboolean delete_event (GtkWidget*, GdkEvent*, gpointer);
static void wp_switch( GtkWidget *widget, gpointer *data);
static void spinnerChanged(GtkSpinButton *spinbutton, gpointer user_data);
static gboolean timedSwitch(gpointer user_data);

/* Make these global because it's easier that way */
GtkWidget *timeSpinner;
gint appTimer;
GtkWidget *window;

int main( int argc, char *argv[] )
{
	/* Widget declarations */
	GtkWidget *appvbox;
	GtkWidget *apphbox;
	GtkWidget *button;
	GtkWidget *menubar;
	GtkWidget *appmenu;
	GtkWidget *wpmenu;
	GtkWidget *wpswitch;
	GtkWidget *wpquit;
	GtkWidget *wplabel;
	
	/* Start Gtk */
	gtk_init (&argc, &argv);

	/* Create a new window */
	window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
	/* Set some properties */
	gtk_window_set_title(GTK_WINDOW(window), "Xfce Wallpaper Changer");
	gtk_window_set_default_size(GTK_WINDOW(window), 250, 50);
	gtk_window_set_position(GTK_WINDOW(window), GTK_WIN_POS_CENTER);
	gtk_window_set_icon_from_file(GTK_WINDOW(window), "/usr/share/icons/hicolor/48x48/apps/xfwpc_icon.png", NULL);
	gtk_container_set_border_width(GTK_CONTAINER(window), 2);
	
	/* Create a tray icon */
	GtkStatusIcon *trayIcon = gtk_status_icon_new_from_file("/usr/share/icons/hicolor/48x48/apps/xfwpc_icon.png");
	/* Tray icon tooltip */
	gtk_status_icon_set_tooltip(trayIcon, "Xfce Wallpaper Changer");
	/* Tray icon popup menu */
	GtkWidget *menu, *menuItemSwitch, *menuItemExit;
	
	menu = gtk_menu_new();
	menuItemSwitch = gtk_menu_item_new_with_label("Switch!");
	menuItemExit = gtk_menu_item_new_with_label("Exit");
	/* Tray icon menu handlers */
	g_signal_connect(G_OBJECT (menuItemSwitch), "activate", G_CALLBACK (wp_switch), NULL);
	g_signal_connect(G_OBJECT (menuItemExit), "activate", G_CALLBACK (trayExit), NULL);
	/* Attach items to menu */
	gtk_menu_shell_append (GTK_MENU_SHELL (menu), menuItemSwitch);
	gtk_menu_shell_append (GTK_MENU_SHELL (menu), menuItemExit);
	gtk_widget_show_all (menu);
	/* Tray icon mouse event handlers */
	g_signal_connect (GTK_STATUS_ICON (trayIcon), "activate", GTK_SIGNAL_FUNC (trayIconActivated), window);
	g_signal_connect (GTK_STATUS_ICON (trayIcon), "popup-menu", GTK_SIGNAL_FUNC (trayIconPopup), menu);
	/* Hook up the delete event */
	g_signal_connect_swapped(G_OBJECT(window), "delete_event", G_CALLBACK(delete_event), NULL);
	/* And the destroy event */
	g_signal_connect (G_OBJECT (window), "destroy", G_CALLBACK (destroy), NULL);
	
	/* Add a label for the Spinbutton */
	wplabel = gtk_label_new("Minutes between changes: ");
	/* Add the  SpinButton */
	timeSpinner = gtk_spin_button_new_with_range (1, 240, 1);
	/* Add a VBox container */
	appvbox = gtk_vbox_new(FALSE, 2);
	/* and an HBox container */
	apphbox = gtk_hbox_new(FALSE, 2);
    
	/* Build the menubar and menu */
	menubar = gtk_menu_bar_new();
	appmenu = gtk_menu_new();
	wpmenu = gtk_menu_item_new_with_label("Menu");
	wpswitch = gtk_menu_item_new_with_label("Switch!");
	wpquit = gtk_menu_item_new_with_label("Exit");
	gtk_menu_item_set_submenu(GTK_MENU_ITEM(wpmenu), appmenu);
	gtk_menu_shell_append(GTK_MENU_SHELL(appmenu), wpswitch);
	gtk_menu_shell_append(GTK_MENU_SHELL(appmenu), wpquit);
	gtk_menu_shell_append(GTK_MENU_SHELL(menubar), wpmenu);
    
	/* Make it work */
	g_signal_connect(G_OBJECT (wpswitch), "activate", G_CALLBACK (wp_switch), NULL);
	g_signal_connect(G_OBJECT (wpquit), "activate", G_CALLBACK (trayExit), NULL);

	/* Pack the vbox */
	gtk_container_add (GTK_CONTAINER (window), appvbox);

	/* Pack the hbox */
	gtk_box_pack_start(GTK_BOX(appvbox), menubar, FALSE, FALSE, 3);
	gtk_box_pack_start(GTK_BOX(appvbox), apphbox, TRUE, TRUE, 2);
	gtk_box_pack_start(GTK_BOX(apphbox), wplabel, TRUE, TRUE, 2);
	gtk_box_pack_start(GTK_BOX(apphbox), timeSpinner, TRUE, TRUE, 2);
    
	/* Read the config file */
	unsigned int startduration = 0;
	FILE *configfile;
	char * configfilename = getenv("HOME");
	strcat (configfilename,"/.xfwpcconf");
	
	if ((configfile = fopen(configfilename, "r")) == NULL)
	{
		/* No file */
		g_print("No file?");
		startduration = 30;
	}
	else
	{
		fscanf(configfile, "%u", &startduration);
		fclose(configfile);
	}
	/* Set the Spinbutton before connecting it to avoid having it throw an error */
	gtk_spin_button_set_value(GTK_SPIN_BUTTON(timeSpinner), (gdouble)startduration);
	/* Convert the ms to seconds for the timer */
	startduration = startduration * 60000;
	
	/* Connect the SpinButton */
	g_signal_connect(G_OBJECT (timeSpinner), "value-changed", G_CALLBACK (spinnerChanged), NULL);
	
	/* The timer */
	appTimer = g_timeout_add(startduration, timedSwitch, NULL);
    
	/* Make the window happen but hide it */
	gtk_widget_show_all (window);
	gtk_widget_hide (window);
  
	/* Start the main loop up */
	gtk_main ();

	return 0;
}

static void trayExit(GtkMenuItem *item, gpointer user_data)
{
	/* Write the config file */
	FILE *configfileout;
	char * configfilenameout = "";
	configfilenameout = getenv("HOME");
	if ((configfileout = fopen(configfilenameout, "w")) == NULL)
	{
		/* No file */
		g_print("No file?");
	}
	else
	{
		fprintf(configfileout, "%d", gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(timeSpinner)));
		fclose(configfileout);
	}
	

	gtk_main_quit();
}

static void trayIconActivated(GObject *trayIcon, gpointer window)
{
	/* If the window's showing, hide it, if hidden, show it */
	if (!gtk_widget_get_visible(GTK_WIDGET(window)))
	{
		gtk_widget_show(GTK_WIDGET(window));
		gtk_window_deiconify(GTK_WINDOW(window));
	}
	else
	{
		gtk_widget_hide(GTK_WIDGET(window));
		gtk_window_iconify(GTK_WINDOW(window));
	}
}

static void trayIconPopup(GtkStatusIcon *status_icon, guint button, guint32 activate_time, gpointer popUpMenu)
{
	gtk_menu_popup(GTK_MENU(popUpMenu), NULL, NULL, gtk_status_icon_position_menu, status_icon, button, activate_time);
}

static void destroy( GtkWidget *widget, gpointer data)
{
	/* Write the config file */
	FILE *configfileout;
	char * configfilenameout = "";
	configfilenameout = getenv("HOME");
	if ((configfileout = fopen(configfilenameout, "w")) == NULL)
	{
		/* No file */
		g_print("No file?");
	}
	else
	{
		fprintf(configfileout, "%d", gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(timeSpinner)));
		fclose(configfileout);
	}
	
	
	/* Kill the app */
	gtk_main_quit();
}

static gboolean delete_event( GtkWidget *widget, GdkEvent *event, gpointer data)
{
	/* Return false here so the app fires destroy() */
		gtk_widget_hide(GTK_WIDGET(window));
		gtk_window_iconify(GTK_WINDOW(window));
	return TRUE;
}

/* Wallpaper switching function */
static void wp_switch( GtkWidget *widget, gpointer *data)
{
	/* Fork another process and run xfdesktop --reload */
	pid_t mypid;
	int mystatus;
	
	mypid = fork();
	
	if (mypid == 0)
	{
		execlp("xfdesktop", "xfdesktop", "--reload", NULL);
	}
	else
	{
		wait(&mystatus);
	}
}

static void spinnerChanged(GtkSpinButton *spinbutton, gpointer user_data)
{
	/* Work that timer! */
	gint tempfloat;
	guint newduration;
	tempfloat = gtk_spin_button_get_value_as_int(spinbutton);
	newduration = tempfloat * 60000;
	g_source_remove(appTimer);
	appTimer = g_timeout_add(newduration, timedSwitch, NULL);
}

/* Switch wallpaper when the timer fires */
static gboolean timedSwitch(gpointer user_data)
{
	pid_t mypid;
	int mystatus;
	
	mypid = fork();
	
	if (mypid == 0)
	{
		execlp("xfdesktop", "xfdesktop", "--reload", NULL);
	}
	else
	{
		wait(&mystatus);
	}
	return TRUE;
}